(******************************************************************************

  ProtoVeriPhy

  Module: distance/SystemSimps.thy

  Copyright (C) 2007-2012 Benedikt Schmidt <benedikt.schmidt@inf.ethz.ch>
  License: BSD (see COPYRIGHT)

******************************************************************************)

theory SystemSimps imports SystemInvariants begin


text {*
        We now define simplifications for the protocol rule for some important
        subclasses of protocols:
          1. executable protocols:
              - do not need the "m : derivMessagesI (Honest A) tr" in the assumptions
              - the view can be simplified to: "[(t + clocktime A,ev) . (t, ev) <- tr]"
          2. time invariant protocols: time translation can also be removed from view
     *}

text {* we need the additional sys parameter because the inductive set sys defined
        in the imported protocol locale is not available in the locale declaration:
        (see C. Ballarin: Tutorial to Locales and Locale Interpretation)
         so we give a (possibly) different sys parameter here and also
         can't use derivMessagesI here *}

locale PROTOW =
  fixes sys_param :: "'msg trace set"

locale PROTOCOL_EXECUTABLE = pe: PROTOCOL _ _ _ _ _ _ _ _ _ _ _ _ Key + PROTOW sys_param
  for sys_param :: "'msg trace set" and Key :: "nat \<Rightarrow> 'msg" +
  assumes messages_derivable:
    "\<lbrakk> step \<in> proto; (m :: 'msg,pEv) \<in> step (view A tr) A (clocktime A t) \<rbrakk> \<Longrightarrow>
       m \<in> DM (Honest A) (knows (Honest A) tr \<union> initState (Honest A))"
  assumes events_occur_at:
    "[| tr \<in> sys_param; step \<in> proto |] ==> step (view A tr) A t = step (timetrans A tr) A t"

lemma (in PROTOCOL_EXECUTABLE) messages_derivableI:
    "\<lbrakk> step \<in> proto; (m :: 'msg,pEv) \<in> step (view A tr) A (clocktime A t) \<rbrakk> \<Longrightarrow>
       m \<in> DM (Honest A) (knowsI (Honest A) (tr::'msg trace))"
  apply (auto simp add: knowsI_def)
  apply (erule messages_derivable)
  apply force
done

lemma (in PROTOCOL_EXECUTABLE) derivable_removable:
   "(\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t;
      step \<in> proto; (m,pEv) \<in> step (view A tr) A (clocktime A t);
      m \<in> DM (Honest A) (knowsI (Honest A) tr) \<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))
   \<equiv>
   (\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t;
      step \<in> proto; (m,pEv) \<in> step (view A tr) A (clocktime A t)\<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))"
  apply  (rule Pure.equal_intr_rule)
  apply (subgoal_tac "m \<in> DM (Honest A) (knowsI (Honest A) tr)")
  apply auto
  apply (rule messages_derivableI)
by auto

lemma (in PROTOCOL_EXECUTABLE) remove_occursAt:
   "(\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t;
      step \<in> proto; (m,pEv) \<in> step (view A tr) A (clocktime A t) \<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))
    ==
    (\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t;
      step \<in> proto; (m,pEv) \<in> step (timetrans A tr) A (clocktime A t) \<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))"
  apply  (rule Pure.equal_intr_rule)
  apply (auto simp add: events_occur_at)
done

end